/************************************************
 *
 *  Uc Agnostic Variables
 *
 *
 ************************************************/

enum gamestate {
  NEW_GAME,
  FORTH,
  BACK,
  PRESSP1,
  PRESSP2,
  P1SCORED,
  P2SCORED,
  WINP1,
  WINP2,
  DISPLAY_SCORE
};

int CurrentGameState = NEW_GAME;
int Player1PressedButton = 0;
int Player2PressedButton = 0;
int globpause = 100;
int scoreP1 = 0;
int scoreP2 = 0;
int lastScored =0;
volatile unsigned int timercnt=0;

/************************************************
 *
 *
 * Uc specific Functions
 *
 *
 ************************************************/

/************************************************
 *  MCS51 / Cypress FX2 specific 
 ************************************************/
#ifdef UC_MCS51
#include <fx2regs.h>
#include <fx2macros.h>
#include <fx2ints.h>
#define SPIIOPORT IOD
#define SPIOEPORT OED
#define	NOP		__asm nop __endasm
#define	SYNCDELAY	NOP; NOP; NOP; NOP

#define CLOCKIT  SPIIOPORT &= (~0x2);SPIIOPORT |= (0x2);
#define LATCHHI  SPIIOPORT |= 0x4;
#define LATCHLO  SPIIOPORT &= (~0x4);
#define DATAHI  SPIIOPORT |= 0x1;
#define DATALO  SPIIOPORT &= (~0x1);

inline void UcSpecificSetup(){
  SETCPUFREQ(CLK_24M);
  SPIOEPORT=0x7;  //enable the pins
  SPIIOPORT=0;
  OEA =0x4; //enable the pins, used as input but OE
  // usually output enable works. 
  IOA = 0 ;
  TMOD = 0x1; 
  TF0=0;
  CKCON|=(1<<3); // fast clock for timers (/4)
  EA=1; // enable interrupts
   IE|=5 ; // enable int0 and int1
  ET0 = 1;
  ENABLE_TIMER0();
  TR0=1; // start t0
  IP|=2; //timer 0 high priority
  IP&=~1; // external0 low priority both important or button press P1 
  //will block advance of the led
}

inline void DisableP1Interrupt(){IE&=~1;}
inline void DisableP2Interrupt(){IE&=~4;}
inline void EnableP1Interrupt(){IE|=1;}
inline void EnableP2Interrupt(){IE|=4;}

static void timer0_isr(void) __interrupt TF0_ISR __using 1 {
  // in MODE 1 timer ticked very CLOCK/4 @ 24/4 = 6 Mhz
  // 6MHZ / 0xffff = 6.10^6 / 65536 = 91.55Hz
  // with a preload val of 5536 should be @ 100Hz (eases the calculations)
  // 5535 = 0x159f
  // with a preload val of 59535 should be @ 1000Hz (eases the calculations)
  // 59353 = 0xe88f
  
  TR0=0;
  TL0=0x8f;
  TH0=0xe8;
  timercnt++;
  //  LEDIOPORT =timercnt;
  // if(timercnt%100 == 0) IOA^=4;
  TR0=1;
}

static void int0_isr(void) __interrupt IE0_ISR __using 1 {
 Player1PressedButton = 1;

}

static void int1_isr(void) __interrupt IE1_ISR __using 1 {
  Player2PressedButton = 1;
}

int getRand(){
  return 1;
}
#endif

/************************************************
 *  MSP430 
 ************************************************/
#ifdef UC_MSP430
#include <legacymsp430.h>

#define PIN_DATA BIT0
#define PIN_CLOCK BIT1
#define PIN_LATCH BIT2

#define PIN_PLAYER1 BIT4
#define PIN_PLAYER2 BIT5

#define SPIPORTDIR P2DIR
#define SPIPORTOUT P2OUT

#define CLOCKIT  SPIPORTOUT|=PIN_CLOCK;SPIPORTOUT&=~PIN_CLOCK;


#define DATAHI  SPIPORTOUT|=PIN_DATA
#define DATALO  SPIPORTOUT&=~PIN_DATA

#define LATCHHI  SPIPORTOUT|=PIN_LATCH
#define LATCHLO  SPIPORTOUT&=~PIN_LATCH

#define PLAYERPORTDIR P1DIR
#define PLAYERPORTIN P1IN
#define PLAYERPORTIE P1IE
#define PLAYERPORT_VECTOR PORT1_VECTOR
#define PLAYERPORT_ISR port1_isr
#define PLAYERPORTIFG P1IFG
volatile unsigned int nmicnt;
volatile unsigned char waitingADC = 1;

interrupt(ADC10_VECTOR) ADC_isr(void){
  waitingADC =0;
}

interrupt(NMI_VECTOR) nmi_isr(void){ /* HERE = PANIC !*/
  if(IFG1 & OFIFG){
    do {
      IFG1 &= ~OFIFG;
      for(nmicnt= 0xff;nmicnt>0;nmicnt--);
      P1OUT ^= 0x01;
      
    } while (IFG1 & OFIFG);
    IE1 |= OFIE;
    P1OUT &= 0x01;
  }
}

interrupt (TIMER0_A0_VECTOR) TimerA0_isr (void){
  timercnt++;
}

interrupt (PLAYERPORT_VECTOR) PLAYERPORT_ISR (void){
  if(PLAYERPORTIFG & PIN_PLAYER1){
    Player1PressedButton=1;
    // do {
    P1IES^= PIN_PLAYER1; // change fall/raise sense
    PLAYERPORTIFG &= ~PIN_PLAYER1;
      //	} while (PLAYERPORTIFG & PIN_PLAYER1); // clear flag
  }else{
    if(PLAYERPORTIFG & PIN_PLAYER2){
	PLAYERPORTIFG &= ~PIN_PLAYER2;
	// do {
	P1IES^= PIN_PLAYER2; // change fall/raise sense
	Player2PressedButton=1;
	 //  } while (PLAYERPORTIFG & PIN_PLAYER2); // clear flag
    }
  }
}

void oscillatorFault(){ /*blinks in case of osc fault*/
  BCSCTL3 = LFXT1S_2; // DEFAULT : XTS = 0 -> uses VLO for LFXT1
  IFG1 &= ~OFIFG; // clear interrupt flg
  BCSCTL2 |= SELM_3|SELS; // MCLK = SMCLK = LFXT1 (see -2lines = VLO)
  
  __bis_status_register(SCG1+SCG0); 
  // stop MCLCK & SMCLK (settingthem vlo => cut power consumption)
  
  while(1){
    P1OUT ^= 0x01;
    __delay_cycles(500); // blink green for problem
  }
}

void initClock(){
  int rep;
  P1OUT |= 0x01;

  DCOCTL  = CALDCO_16MHZ;
  BCSCTL1 = CALBC1_16MHZ;
  /* DEVICE DEPENDENT !!! 
     DCO running @16MHz => internal calibration
     BOTH LINE COUNTS : RSEL are in BCSCTL1 while subfreq in range for
     DCO is in DCOCTL
  */

  BCSCTL1 |= XT2OFF;  /* ACLK = LFXTCLK =  32768hz xtal, /1 prescaler
			XT2 input is OFF
		     */


  BCSCTL2 = 0; // MCLCK = DCO / 1, SMCLK = DCO / 1 
  BCSCTL3 = XCAP_3; /* internal XTAL caps (TI provided : 12.5pf)*/
  rep = 0;
  IFG1 &= ~OFIFG;
  while (IFG1 & OFIFG){ /* loop 10 times if after xtal not stab -> problem*/
    IFG1 &= ~OFIFG;
    __delay_cycles(10000);
    if(!(rep<10))
      oscillatorFault();
    rep++;
  }
  P1OUT = 0x00;
}

void disableWDT(){
  WDTCTL= WDTPW|WDTHOLD; // disable watchdog
}

inline void UcSpecificSetup(){
 disableWDT(); // disable watchdog
  initClock(); // see func
  IE1 |= OFIE;  // enable clock problem interrupt

  SPIPORTDIR |= PIN_DATA|PIN_LATCH|PIN_CLOCK ; // 3 pins for shiftreg

  PLAYERPORTDIR = 0; // default just for show
  P1DIR |= (BIT6 | BIT0); // dir 1 = 1&6 output // integrated leds
  //P1OUT |= BIT0;
  // without config : all in -> no need to config for player input
  PLAYERPORTIE = PIN_PLAYER1|PIN_PLAYER2; // enable interrupt on player pins

  TACTL = TASSEL_1|ID_0|MC_1;
    /* ACLKC = LFXT1 : 32768hz | /1 | Up to CCR0*/
  CCR0 =  0x21;
    // (32768 / 1000 = 32.7 = 33) 
 
  TACCTL0 |= CCIE;
  
  /* setup adc*/
  ADC10CTL0 = SREF_0|ADC10SHT_3|REFON|ADC10ON|ADC10IE;
  ADC10CTL1 = ADC10SSEL_0|INCH_0|ADC10DIV_3;
  eint();
}

inline void DisableP1Interrupt(){PLAYERPORTIE &= ~PIN_PLAYER1;}
inline void DisableP2Interrupt(){PLAYERPORTIE &= ~PIN_PLAYER2;}
inline void EnableP1Interrupt(){PLAYERPORTIE |= PIN_PLAYER1;}
inline void EnableP2Interrupt(){PLAYERPORTIE |= PIN_PLAYER2;}

int getRand(){
  	ADC10CTL0 |= ENC|ADC10SC;
	while(waitingADC);
	if(ADC10MEM&1){
	  return 1;
	}else{
	  return 0;
	}
}

#endif

/************************************************
 * Stellaris Launchpad
 ************************************************/
#ifdef UC_STELLARIS

#include "inc/lm4f120h5qr.h"
#include "inc/hw_ints.h"
#include "inc/hw_types.h"
#include "inc/hw_memmap.h"
#include "driverlib/sysctl.h"
#include "driverlib/gpio.h"
#include "driverlib/adc.h"
#include "driverlib/interrupt.h"
#include "driverlib/timer.h"
//F1 
#define RED_LED   GPIO_PIN_1 
//F2
#define BLUE_LED  GPIO_PIN_2 
//F3
#define GREEN_LED GPIO_PIN_3 
//E3
#define LATCH_PIN GPIO_PIN_3
//E1
#define DATA_PIN GPIO_PIN_1
//E2
#define CLOCK_PIN GPIO_PIN_2
/*For random*/
#define ADC_FLOATING_PIN GPIO_PIN_2
#define PLAYER1_PIN GPIO_PIN_1
#define PLAYER2_PIN GPIO_PIN_0

#define CLOCKIT GPIOPinWrite(GPIO_PORTE_BASE, CLOCK_PIN,0);GPIOPinWrite(GPIO_PORTE_BASE, CLOCK_PIN,CLOCK_PIN);


#define DATAHI  GPIOPinWrite(GPIO_PORTE_BASE, DATA_PIN,DATA_PIN);
#define DATALO  GPIOPinWrite(GPIO_PORTE_BASE, DATA_PIN,0);

#define LATCHHI GPIOPinWrite(GPIO_PORTE_BASE, LATCH_PIN,LATCH_PIN);
#define LATCHLO GPIOPinWrite(GPIO_PORTE_BASE, LATCH_PIN,0);

inline void DisableP1Interrupt(){  GPIOPinIntDisable(GPIO_PORTD_BASE,PLAYER1_PIN);}
inline void DisableP2Interrupt(){  GPIOPinIntDisable(GPIO_PORTD_BASE,PLAYER2_PIN);}
inline void EnableP1Interrupt(){GPIOPinIntClear(GPIO_PORTD_BASE,PLAYER1_PIN|PLAYER2_PIN);GPIOPinIntEnable(GPIO_PORTD_BASE,PLAYER1_PIN);}
inline void EnableP2Interrupt(){GPIOPinIntClear(GPIO_PORTD_BASE,PLAYER1_PIN|PLAYER2_PIN);GPIOPinIntEnable(GPIO_PORTD_BASE,PLAYER2_PIN);}

void Timer0IntHandler(void)
{
// Clear the timer interrupt 
  TimerIntClear(TIMER0_BASE, TIMER_TIMA_TIMEOUT);
  timercnt++;
}


inline void UcSpecificSetup(){
  unsigned int ulPeriod;

SysCtlClockSet(SYSCTL_SYSDIV_12_5|SYSCTL_USE_PLL|SYSCTL_XTAL_16MHZ|
                    SYSCTL_OSC_MAIN);

//
    // Enable and configure the GPIO port for the LED operation.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOF);
    GPIOPinTypeGPIOInput(GPIO_PORTD_BASE, PLAYER1_PIN|PLAYER2_PIN);
    GPIOPinTypeGPIOOutput(GPIO_PORTE_BASE, LATCH_PIN|DATA_PIN|CLOCK_PIN);
    GPIOPinTypeGPIOOutput(GPIO_PORTF_BASE, BLUE_LED|GREEN_LED|RED_LED);

    /*
      TIMER 0A setup
     */
    SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER0);
    TimerConfigure(TIMER0_BASE, TIMER_CFG_32_BIT_PER);
    ulPeriod = (SysCtlClockGet() / (1000));
    TimerLoadSet(TIMER0_BASE, TIMER_A, ulPeriod -1);
    IntEnable(INT_TIMER0A);
    TimerIntEnable(TIMER0_BASE, TIMER_TIMA_TIMEOUT);
    IntMasterEnable();
    TimerEnable(TIMER0_BASE, TIMER_A);

    /*
      ADC SETUP...it's a bit convoluted but ADC stepping "poney r0x" !
     */

    GPIOPinTypeADC(GPIO_PORTD_BASE,ADC_FLOATING_PIN);//D3

   
    SysCtlPeripheralEnable(SYSCTL_PERIPH_ADC0);
    SysCtlADCSpeedSet(SYSCTL_ADCSPEED_250KSPS);
    // disable before conf
    ADCSequenceDisable(ADC0_BASE, 1);
    ADCSequenceConfigure(ADC0_BASE, 1, ADC_TRIGGER_PROCESSOR, 0);
    ADCSequenceStepConfigure(ADC0_BASE, 1, 0, ADC_CTL_CH5); 
    // CH4 = AIN4 = D3, datasheet p762 
    // AND ADCSSMUX REGISTER  p810 + see ADCSequenceStepConfigure source in adc.c
    ADCSequenceStepConfigure(ADC0_BASE, 1, 1, ADC_CTL_CH5);
    ADCSequenceStepConfigure(ADC0_BASE, 1, 2, ADC_CTL_CH5);
    ADCSequenceStepConfigure(ADC0_BASE, 1, 3, ADC_CTL_CH5 | ADC_CTL_IE | ADC_CTL_END);
    ADCSequenceEnable(ADC0_BASE, 1);
    /*setup interrupts*/
    IntEnable(INT_GPIOD);
    GPIOIntTypeSet(GPIO_PORTD_BASE,PLAYER1_PIN|PLAYER2_PIN,GPIO_BOTH_EDGES);
    GPIOPadConfigSet(GPIO_PORTD_BASE,PLAYER1_PIN|PLAYER2_PIN,GPIO_STRENGTH_2MA,GPIO_PIN_TYPE_STD);
    IntMasterEnable();
}

int getRand(){
 unsigned long ulADC0Value[4];
ADCIntClear(ADC0_BASE, 1);
	ADCProcessorTrigger(ADC0_BASE, 1);
	while(!ADCIntStatus(ADC0_BASE, 1, false))
	  {
	  }
	ADCSequenceDataGet(ADC0_BASE, 1, ulADC0Value);
return ulADC0Value[0];
}

void mISRGPIOD(void){
  if(GPIO_PORTD_RIS_R & PLAYER1_PIN){
  GPIOPinIntClear(GPIO_PORTD_BASE,PLAYER1_PIN);
  Player1PressedButton=1;
  }else{
  GPIOPinIntClear(GPIO_PORTD_BASE,PLAYER2_PIN);
  Player2PressedButton=1;
  }
}

#endif

/************************************************
 * ST F4 Discovery
 ************************************************/

#ifdef UC_STM32F4
#include "stm32f4_discovery.h"
#include "stm32f4xx_conf.h" // again, added because ST didn't put it here ?
#include "stm32f4xx_rng.h"
#include <peripherals/stm32f4xx_adc.h>
#include <stdbool.h>
#define PORT595 GPIOB
#define RCC_AHB1Periph_PORT595 RCC_AHB1Periph_GPIOB
#define LATCH_PIN GPIO_Pin_0
#define DATA_PIN GPIO_Pin_1
#define CLOCK_PIN GPIO_Pin_2

#define PORTLED GPIOD
#define RCC_AHB1Periph_PORTLED RCC_AHB1Periph_GPIOD
#define PIN_LED_R GPIO_Pin_14
#define PIN_LED_G GPIO_Pin_12
#define PIN_LED_B GPIO_Pin_15
#define PIN_LED_O GPIO_Pin_13
#define PIN_LED_R2 GPIO_Pin_5

#define PORTLEDAUX GPIOA
#define RCC_AHB1Periph_PORTLEDAUX RCC_AHB1Periph_GPIOA
#define PIN_LED_G_AUX GPIO_Pin_9

#define PORTBTN GPIOC
#define RCC_AHB1Periph_PORTBTN RCC_AHB1Periph_GPIOC
#define PLAYER1_PIN GPIO_Pin_0
#define PLAYER2_PIN GPIO_Pin_1
#define EXTI_PortSource_PORTBTN EXTI_PortSourceGPIOC

GPIO_InitTypeDef  GPIO_InitStructure;
TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
NVIC_InitTypeDef   NVIC_InitStructure;
EXTI_InitTypeDef   EXTI_InitStructure;

#define CLOCKIT GPIO_SetBits(PORT595, CLOCK_PIN);GPIO_ResetBits(PORT595, CLOCK_PIN);


#define DATAHI GPIO_SetBits(PORT595,DATA_PIN); 
#define DATALO GPIO_ResetBits(PORT595,DATA_PIN);

#define LATCHHI  GPIO_SetBits(PORT595,LATCH_PIN); 
#define LATCHLO GPIO_ResetBits(PORT595, LATCH_PIN);

int getRand(){
  while(!RNG_GetFlagStatus(RNG_FLAG_DRDY))
    {
      ;
    }
  // return 1;
  return  RNG_GetRandomNumber();
}

void SysTick_Handler(void)
{
  timercnt++;
}

void EXTI0_IRQHandler(void)
{
      
         Player1PressedButton=1;
         EXTI_ClearITPendingBit(EXTI_Line0);
 
      
}
void EXTI1_IRQHandler(void)
{
   
         Player2PressedButton=1;
         EXTI_ClearITPendingBit(EXTI_Line1);
       
}

inline void UcSpecificSetup(){
 /*!< At this stage the microcontroller clock setting is already configured, 
       this is done through SystemInit() function which is called from startup
       file (startup_stm32f4xx.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
        system_stm32f4xx.c file
     */
  /* GPIOs Periph clock enable */
  GPIO_InitTypeDef  GPIO_InitStructure2;
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_PORTLED, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_PORTLEDAUX, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_PORTBTN, ENABLE);  
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_PORT595, ENABLE);
  RCC_AHB2PeriphClockCmd(RCC_AHB2Periph_RNG, ENABLE);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
  RNG_Cmd(ENABLE);
 

 if (SysTick_Config(SystemCoreClock / 1000))
  { 
    /* Capture error */ 
    while (1);
  }


  GPIO_InitStructure.GPIO_Pin = PIN_LED_R|PIN_LED_R2|PIN_LED_G|PIN_LED_B|PIN_LED_O;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(PORTLED, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = LATCH_PIN |DATA_PIN|CLOCK_PIN;
  GPIO_Init(PORT595, &GPIO_InitStructure);

  /*  GPIO_InitStructure.GPIO_Pin =PIN_LED_G_AUX;
  GPIO_Init(PORTLEDAUX, &GPIO_InitStructure);
  */
  
  GPIO_InitStructure2.GPIO_Pin = (GPIO_Pin_0|GPIO_Pin_1);
  GPIO_InitStructure2.GPIO_Mode = 0;
  GPIO_InitStructure.GPIO_OType = 0;
  GPIO_InitStructure2.GPIO_PuPd= 0;
  GPIO_Init(PORTBTN, &GPIO_InitStructure2);

  GPIO_ResetBits(PORTLED, PIN_LED_R2); // strangely enabled by default ?

  SYSCFG_EXTILineConfig(EXTI_PortSource_PORTBTN, EXTI_PinSource0);
  
  /* Configure EXTI Line0 */
  EXTI_InitStructure.EXTI_Line = EXTI_Line0;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising_Falling;  
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  EXTI_Init(&EXTI_InitStructure);

  SYSCFG_EXTILineConfig(EXTI_PortSource_PORTBTN, EXTI_PinSource1);
  EXTI_InitStructure.EXTI_Line = EXTI_Line1;
  EXTI_Init(&EXTI_InitStructure);

  /* Enable and set EXTI Line0 Interrupt to the lowest priority */
  NVIC_InitStructure.NVIC_IRQChannel = EXTI0_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x01;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x01;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  NVIC_InitStructure.NVIC_IRQChannel = EXTI1_IRQn;
  NVIC_Init(&NVIC_InitStructure);
  int first = 1;

}
inline void DisableP1Interrupt(){;}
inline void DisableP2Interrupt(){;}
inline void EnableP1Interrupt(){;}
inline void EnableP2Interrupt(){;}
#endif


/************************************************
 * Catch of sometime not defined vars
 ************************************************/

#ifndef PLAYER1_PIN
#define PLAYER1_PIN 0
#endif

#ifndef PLAYER2_PIN
#define PLAYER2_PIN 1
#endif



/************************************************
 *
 *  Uc Agnostic functions
 *
 *
 ************************************************/

void DelayMs(int ms){
  timercnt=0;
  while(timercnt<ms){
    ;
  }
}

int dec2bin(int dec){
  int ret = 0;
  while(dec){
    ret<<=1;
    ret|=1;
    dec--;
  }
  return ret;
} 

void cleanLed(unsigned int i){
  while(i--){
    CLOCKIT;
  }
}

void ledforth(int pause){
unsigned char i=0;
 LATCHLO;
  DATAHI;
  CLOCKIT;
  LATCHHI;
  DATALO;
  for(i=0;i<56;i++){
    DelayMs(pause);
    LATCHLO;
    CLOCKIT;
    LATCHHI;
      if(i==53){	

	EnableP2Interrupt();
	Player2PressedButton=0;
      }
  }
  if(Player2PressedButton){
    CurrentGameState=BACK;
  }else{
    CurrentGameState=P1SCORED;
  }
  DisableP2Interrupt();

}

void ledback(unsigned long pause){
  int i,j;
  for(j=56;j>=0;j--){
    cleanLed(56);
    i=0;
    if(j==4){
      //interrupt press P1
    }
    LATCHLO;
    DATAHI;
    //digitalWrite(dataPin,HIGH);
    CLOCKIT;
    DATALO;
    for(i=0;i<j;i++){
       CLOCKIT;

    }
      if(j==3){
	EnableP1Interrupt();
	Player1PressedButton=0;
      }
    LATCHHI;
    DelayMs(pause);
  }  
  if(Player1PressedButton){
    CurrentGameState=FORTH;
  }else{
    CurrentGameState=P2SCORED;
  }
  DisableP1Interrupt();
}

void nextglobpause(){
  globpause *=0.9;
  if(globpause < 10){
  globpause = 10;
  }
}

void displayScore(int score1, int score2){
  int i = 0 ,locscore2;
  cleanLed(56);
  locscore2=dec2bin(score2);
  LATCHLO;

  DATAHI;

  while(locscore2){
    locscore2>>=1;
    CLOCKIT;
    i++;
  }
  DATALO;
  while((i+score1)<56){
    CLOCKIT;
    i++;
  }
  DATAHI;

  while(i<56){
    CLOCKIT;
    i++;
  }
  LATCHHI;
    DATALO;
}

int main(void)
{
  unsigned char i=0;
  UcSpecificSetup();
   while (1)
  {
   switch(CurrentGameState){
      case NEW_GAME:
	if( getRand()&1){ //
	  CurrentGameState=FORTH;
	}else{
	  CurrentGameState=BACK;
	}
	break;
      case FORTH:
	nextglobpause();
	cleanLed(56);
	ledforth(globpause);
	
	break;
      case BACK:
	nextglobpause();
	ledback(globpause);
	
	break;
      case P1SCORED: 
	scoreP1++;
	lastScored=PLAYER1_PIN;
	CurrentGameState=DISPLAY_SCORE;
	break;
      case P2SCORED: 
	scoreP2++;
	lastScored=PLAYER2_PIN;
	CurrentGameState=DISPLAY_SCORE;
	break;
      case DISPLAY_SCORE:
	displayScore(scoreP1,scoreP2);
	DelayMs(500);
	if(lastScored==PLAYER1_PIN){
	  CurrentGameState=BACK;
	}else{
	  CurrentGameState=FORTH;
	}
	if((scoreP1+scoreP2)>10){
	  if(scoreP1>scoreP2){
	    CurrentGameState=WINP1;
	  }else{
	    CurrentGameState=WINP2;
	  }
	}
	break;
      case WINP1:
	while(1){
	  displayScore(scoreP1,scoreP2);
	  DelayMs(200);
	  displayScore(scoreP1,0);
	  DelayMs(200);
	}
	break;
      case WINP2:
	while(1){
	  displayScore(scoreP1,scoreP2);
	  DelayMs(200);
	  displayScore(0,scoreP2);
	  DelayMs(200);
	}
	break;
      }
      Player1PressedButton=Player2PressedButton=0;

  }
}
